package Data::Transpose::Validator::EmailValid;

use strict;
use warnings;
use base 'Data::Transpose::EmailValid';

=head1 NAME

Data::Transpose::Validator::EmailValid - Alias for L<Data::Transpose::EmailValid>

=cut



1;
