package Data::Transpose::Validator::PasswordPolicy;
# wrapper for Data::Transpose::PasswordPolicy

use strict;
use warnings;
use base 'Data::Transpose::PasswordPolicy';

=head1 NAME

Data::Transpose::Validator::PasswordPolicy - Alias for Data::Transpose::PasswordPolicy

=cut



1;
